/*
 * Copyright 2019 European Commission
 *
 * Licensed under the EUPL, Version 1.2 or – as soon they will be approved by the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 *     https://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and limitations under the Licence.
 */
/**
 * @fileOverview 
 * This plugin was customized from the official CKEditor_4.7.1 showblocks plugin to apply different styles and to change the list of tags on which this plugin
 * takes action.
 * Here are the main customizations:
 *     - Changed the pluginName from showblocks to leosShowblocks
 *     - Changed the supported elements/tags 
 *     - Removed "lang" and changed "icons" from the plugin definition
 *     - Removed the function "AddCss"
 */
; // jshint ignore:line
define(function leosShowblocksPluginModule(require) {
    "use strict";

    // load module dependencies
    var pluginTools = require("plugins/pluginTools");
    var pluginName = "leosShowblocks";

    // TODO implement translations
    var supportedElements = [{
        key: 'div'
    }, {
        key: 'div p'
    }, {
        key: 'ul'
    }, {
        key: 'ol'
    }, {
        key: 'ul li'
    }, {
        key: 'ol li'
    }];
    var pluginDefinition = {
        icons: pluginName.toLowerCase(), // %REMOVE_LINE_CORE%

        init: function init(editor) {

            if (editor.blockless) {
                return;
            }

            var command = editor.addCommand('leosShowBlocks', commandDefinition);
            command.canUndo = false;

            if (editor.config.startupOutlineBlocks) {
                command.setState(CKEDITOR.TRISTATE_ON);
            }

            editor.ui.addButton && editor.ui.addButton('LeosShowBlocks', {
                label: 'Show blocks',
                command: 'leosShowBlocks',
                toolbar: 'tools'
            });

            // Refresh the command on setData.
            editor.on('mode', function() {
                if (command.state != CKEDITOR.TRISTATE_DISABLED) {
                    command.refresh(editor);
                }
            });

            // Refresh the command on setData.
            editor.on('contentDom', function(event) {
                if (command.state != CKEDITOR.TRISTATE_DISABLED) {
                    command.refresh(editor);
                }
            });
        }
    };

    var commandDefinition = {
        readOnly: 1,
        preserveState: true,
        editorFocus: false,

        exec: function executeCommandDefinition(editor) {
            this.toggleState();
            this.refresh(editor);
            editor.fire("contentChange");
        },

        refresh: function refreshCommandDefinition(editor) {
            if (editor.document) {
                // Code removed from plugin to avoid switching off show blocks on focus lost.
                var funcName = ( this.state == CKEDITOR.TRISTATE_ON ) ? 'attachClass' : 'removeClass';
                editor.editable()[funcName]('cke_show_blocks');
            }
        }
    };

    pluginTools.addPlugin(pluginName, pluginDefinition);

    // return plugin module
    var pluginModule = {
        name: pluginName
    };

    return pluginModule;
});
